@@ -6,13 +6,14 @@ Module::Runtime - runtime module handling
 
 	use Module::Runtime qw(
 		$module_name_rx is_module_name check_module_name
-		require_module
+		module_notional_filename require_module
 	);
 
 	if($module_name =~ /\A$module_name_rx\z/o) { ...
 	if(is_module_name($module_name)) { ...
 	check_module_name($module_name);
 
+	$notional_filename = module_notional_filename($module_name);
 	require_module($module_name);
 
 	use Module::Runtime qw(use_module use_package_optimistically);
@@ -49,12 +50,12 @@ use strict;
 
 use Params::Classify 0.000 qw(is_string);
 
-our $VERSION = "0.008";
+our $VERSION = "0.009";
 
 use parent "Exporter";
 our @EXPORT_OK = qw(
 	$module_name_rx is_module_name is_valid_module_name check_module_name
-	require_module
+	module_notional_filename require_module
 	use_module use_package_optimistically
 	$top_module_spec_rx $sub_module_spec_rx
 	is_module_spec is_valid_module_spec check_module_spec
@@ -152,6 +153,29 @@ sub check_module_name($) {
 	}
 }
 
+=item module_notional_filename(NAME)
+
+Generates a notional relative filename for a module, which is used in
+some Perl core interfaces.
+The I<NAME> is a string, which should be a valid module name (one or
+more C<::>-separated segments).  If it is not a valid name, the function
+C<die>s.
+
+The notional filename for the named module is generated and returned.
+This filename is always in Unix style, with C</> directory separators
+and a C<.pm> suffix.  This kind of filename can be used as an argument to
+C<require>, and is the key that appears in C<%INC> to identify a module,
+regardless of actual local filename syntax.
+
+=cut
+
+sub module_notional_filename($) {
+	&check_module_name;
+	my($name) = @_;
+	$name =~ s!::!/!g;
+	return $name.".pm";
+}
+
 =item require_module(NAME)
 
 This is essentially the bareword form of C<require>, in runtime form.
@@ -171,15 +195,7 @@ was already loaded.
 =cut
 
 sub require_module($) {
-	&check_module_name;
-	my($name) = @_;
-	# This translation to Unix-style filename is correct regardless
-	# of platform.  This is what ck_require() in the Perl core does
-	# with a bareword, and pp_require() translates the Unix-style
-	# filename to whatever is appropriate for the real platform.
-	$name =~ s!::!/!g;
-	$name .= ".pm";
-	return require($name);
+	return require(&module_notional_filename);
 }
 
 =back
@@ -223,23 +239,15 @@ uncertainty as to whether a package/class is defined in its own module
 or by some other means.  It attempts to arrange for the named package to
 be available, either by loading a module or by doing nothing and hoping.
 
-If the package does not appear to already be loaded then an attempt is
-made to load the module of the same name (as if by the bareword form
+An attempt is made to load the named module (as if by the bareword form
 of C<require>).  If the module cannot be found then it is assumed that
 the package was actually already loaded but wasn't detected correctly,
 and no error is signalled.  That's the optimistic bit.
 
-For the purposes of this function, package existence is checked by whether
-a C<$VERSION> variable exists in the package.  If the module wasn't found,
-or if it was loaded but didn't create a C<$VERSION> variable, then such a
-variable is automatically created (with value C<undef>) so that repeated
-use of this function won't redundantly attempt to load the module.
-
-This is mostly the same operation that is performed by the L<base>
-pragma to ensure that the specified base classes are available.
-The difference is that L<base> does not allow the C<$VERSION> variable
-to remain undefined: it will set it to "C<-1, set by base.pm>" if it does
-not otherwise have a non-null value.
+This is mostly the same operation that is performed by the L<base> pragma
+to ensure that the specified base classes are available.  The behaviour
+of L<base> was simplified in version 2.18, and this function changed
+to match.
 
 If a I<VERSION> is specified, the C<VERSION> method of the loaded package is
 called with the specified I<VERSION> as an argument.  This normally serves
@@ -249,24 +257,14 @@ function work just like L</use_module>.
 
 =cut
 
-sub _has_version_var($) {
-	my($name) = @_;
-	no strict "refs";
-	my $vg = ${"${name}::"}{VERSION};
-	return $vg && *{$vg}{SCALAR};
-}
-
 sub use_package_optimistically($;$) {
 	my($name, $version) = @_;
 	check_module_name($name);
-	unless(_has_version_var($name)) {
-		eval "local \$SIG{__DIE__}; require $name";
-		die $@ if $@ ne "" && $@ !~ /\ACan't locate .* at \(eval /;
-		unless(_has_version_var($name)) {
-			no strict "refs";
-			${"${name}::VERSION"} = undef;
-		}
-	}
+	eval { local $SIG{__DIE__}; require(module_notional_filename($name)); };
+	die $@ if $@ ne "" && $@ !~ /\A
+		Can't\ locate\ .+\ at
+		\ @{[__FILE__]}\ line\ @{[__LINE__-1]}
+	/xs;
 	$name->VERSION($version) if defined $version;
 	return $name;
 }
